﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// 랴디
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 스트립 카운트
        /// </summary>
        private int STRIP_COUNT = 10;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.sourcePictureBox.Paint += pictureBox_Paint;
            this.targetPictureBox.Paint += pictureBox_Paint;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 파일 열기 메뉴 클릭시 처리하기 - openFileMenuItem_Click(sender, e)

        /// <summary>
        /// 파일 열기 메뉴 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void openFileMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog { Filter = "Image|*.png;*.jpeg;*.jpg;*.tif" };

            if(openFileDialog.ShowDialog() == DialogResult.OK)
            {
                ProcessBitmap(openFileDialog.FileName);
            }
        }

        #endregion
        #region 픽처 박스 페인트시 처리하기 - pictureBox_Paint(sender, e)

        /// <summary>
        /// 픽처 박스 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void pictureBox_Paint(object sender, PaintEventArgs e)
        {
            using(var pen = new Pen(Color.FromArgb(20, Color.Black)))
            {
                for(int i = 0; i < 1000; i += 30)
                {
                    e.Graphics.DrawLine(pen, 0, i, 1000, i);
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 비트맵 처리하기 - ProcessBitmap(filePath)

        /// <summary>
        /// 비트맵 처리하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        private void ProcessBitmap(string filePath)
        {
            Bitmap sourceBitmap = Image.FromFile(filePath) as Bitmap;

            this.sourcePictureBox.SourceBitmap = sourceBitmap.Clone() as Bitmap;

            StripInfo stripInfo = new StripInfo(sourceBitmap, STRIP_COUNT);

            int stripX1 = 2;
            int stripX2 = 6;

            double angle = SkewCalculator.GetRotateAngle(stripInfo, stripX1, stripX2);

            angle = (angle * 180 / Math.PI);

            Text = "스캔 이미지 왜곡 자동 보정하기 : 각도 = " + angle.ToString("0.00");

            Bitmap targetBitmap = BitmapHelper.RotateBitmap(sourceBitmap, angle);

            this.targetPictureBox.SourceBitmap = targetBitmap;
        }

        #endregion
    }
}