using System;

namespace TestProject
{
    /// <summary>
    /// SKEW 
    /// </summary>
    public static class SkewCalculator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region ȸ  ϱ - GetRotateAngle(stripInfo, stripX1, stripX2, maximumSkew)

        /// <summary>
        /// ȸ  ϱ
        /// </summary>
        /// <param name="stripInfo">Ʈ </param>
        /// <param name="stripX1">Ʈ X1</param>
        /// <param name="stripX2">Ʈ X2</param>
        /// <param name="maximumSkew">ִ SKEW</param>
        /// <returns></returns>
        public static double GetRotateAngle(StripInfo stripInfo, int stripX1, int stripX2, int maximumSkew = 70)
        {
            double shift  = 0d;
            int    shift1 = FindBestShift(stripInfo, stripX1    , stripX2    , -maximumSkew, maximumSkew);
            int    shift2 = FindBestShift(stripInfo, stripX1 + 1, stripX2 + 1, -maximumSkew, maximumSkew);

            if(Math.Abs(shift1 - shift2) < 2)
            {
                shift = (shift1 + shift2) / 2d;
            }
            else
            {
                if(Math.Abs(shift1) < Math.Abs(shift2))
                {
                    shift = shift1;
                }
                else
                {
                    shift = shift2;
                }
            }

            return Math.Atan2
            (
                shift / stripInfo.Scale,
                (stripX2 - stripX1) * stripInfo.SourceWidth / stripInfo.Width
            );
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region   ϱ - CalculateCorrection(stripInfo, x1, x2, shift)

        /// <summary>
        ///   ϱ
        /// </summary>
        /// <param name="stripInfo">Ʈ </param>
        /// <param name="x1">X1</param>
        /// <param name="x2">X2</param>
        /// <param name="shift">Ʈ</param>
        /// <returns> </returns>
        private static double CalculateCorrection(StripInfo stripInfo, int x1, int x2, int shift)
        {
            int startIndex = shift < 0 ? -shift : 0;
            int endIndex   = shift > 0 ? stripInfo.Height - shift - 1 : stripInfo.Height - 1;

            int[] stripArray1 = stripInfo.StripArray[x1];
            int[] stripArray2 = stripInfo.StripArray[x2];

            int res = 0;

            for(int i = startIndex; i <= endIndex; i++)
            {
                res += -Math.Abs(stripArray1[i] - stripArray2[i + shift]);
            }

            return 1d * stripArray1.Length * res / (stripArray1.Length - Math.Abs(shift));
        }

        #endregion
        #region  Ʈ ã - FindBestShift(stripInfo, x1, x2, startShift, endShift)

        /// <summary>
        ///  Ʈ ã
        /// </summary>
        /// <param name="stripInfo">Ʈ </param>
        /// <param name="x1">X1</param>
        /// <param name="x2">X2</param>
        /// <param name="startShift"> Ʈ</param>
        /// <param name="endShift"> Ʈ</param>
        /// <returns> Ʈ</returns>
        private static int FindBestShift(StripInfo stripInfo, int x1, int x2, int startShift, int endShift)
        {
            double bestCorrection = double.MinValue;

            int bestShift = 0;

            for(int shift = startShift; shift <= endShift; shift++)
            {
                double correction = CalculateCorrection(stripInfo, x1, x2, shift);

                if(correction > bestCorrection)
                {
                    bestCorrection = correction;

                    bestShift = shift;
                }
            }

            return bestShift;
        }

        #endregion
    }
}